;*** Izvorni program: avtor Matjaz Vidmar S53MV / sintetizator frekvence za NBP 10 Mbps ZIF TRX
;*** Original source (assembly code mnemonics written in CAPITALS), author Matjaz Vidmar S53MV: http://lea.hamradio.si/~s53mv/3zif/design.html
;
;*** Priredba (S56AL): Mnozilnik frekvence ure z MAX2870 - S56AL 30.05.2021 ***
;*** Necessary adjustments + some addtional "bells and whistles" added by S56AL (mnemonics written in lowercase):
; - f_ref = 10 MHz,
; - peprogrammed clock out frequencies: 25,000 MHz, 27,000 Mhz, 32,000 MHz, 40,000 MHz, 50,000 MHz
; - Activity/status signal LED: FPLL (re)initialisation ... one blink T = ca. 2s; 
;                          standby ... one tick per 5s (short LED blinks of 0,3s)

;*** Uporaba prikljuckov PIC12F675+WDT Vss(8) Vdd(1) ***
;GP0(7) - CMOS izhod (LE MAX2870)
;GP1(6) - CMOS izhod (DATA MAX2870)
;GP2(5) - CMOS izhod (CLK MAX2870)
;GP3(4) - CMOS vhod (MUXOUT MAX2870)
;GP4(3) - analogni vhod AN3 (izbira frekvence z upori)
;GP5(2) - CMOS izhod (TEST)
;20,21,22,23,24,25 - splosno racunanje
;30 - stari kanal (0FF neveljaven)
;35 - zastavica WDT
;38,39 - casovnik KUZA

;********************************************************************
;*      Mnozilnik frekvence ure z MAX2870                           *
;*      Latest update: S56AL, 18.06.2021                            *
;*                                                                  *
;********************************************************************
	
	SUBTITL "CW_ident"
	LIST     P=12F675

	include "P12F675.INC"       	 ; Vkljucene deklaracije registrov iz datoteke P12F629.inc

;********************************************************************
;*      Configuration bits settings                                 *
;********************************************************************
;
;		Oscillator			Interni
;               CLK pins:			I/O function @ GP4 & GP5
;		WDT				On
;		Power up timer			On
;		MCLR pin:			digital I/O function
;		Brown out detect		On
;		Code protect			Off
;		Data EE read protect		Off
;
; CONFIG
; __config 0x3FCC
 __CONFIG _FOSC_INTRCIO & _WDTE_ON & _PWRTE_ON & _MCLRE_OFF & _BOREN_ON & _CP_OFF & _CPD_OFF

;********************************************************************
;*      Sistemske  konstante                                        *
;********************************************************************

 ;********************************************************************
;*      Periferne naprave (dodelitev I/O)                           *
;********************************************************************
led_act	    equ		5		; LED "activity"

 ;********************************************************************
;*      Data locations in RAM.                                      *
;********************************************************************
;*      Data locations mapped in all four RAM pages = 70h ... h   *
;********************************************************************

;********************************************************************
;*      Spremenljivke RAM page 0 = 20h ... h                      *
;********************************************************************
led_ticker  equ		0x32	; Timer tiktakanja LED "activity"
led_tmo	    equ		0x33	; LED time-out timer (vpis vrednosti N v timer "led_tmo" pri?ge LED za ?as N x 0,3s
	
;********************************************************************
;*      Reset vectors                                               *
;********************************************************************
	org     0x0000
	goto    reset           		; Reset vector
	org     0x0004          		; Interrupt vector
	goto	reset

;********************************************************************
;*      Inicializcija ob resetu                                     *
;********************************************************************
reset
	clrf    STATUS         	 	; Lowest register banks selected
	clrf    PCLATH
	clrf    INTCON
	clrwdt
;--------------------------------------------------------------------
;*		Inicializacija registrov in spremenljivk            *
;--------------------------------------------------------------------
	MOVLW	0x00	;brisi vse izhode GPIO
	MOVWF	0x05
	BSF	0x03,5	;RP0=1 nastavim banko 1 za dostop do nastavitev
	MOVLW	0x18	;TRISIO(85) vhoda GP4,3, izhodi GP5,2,1,0
	MOVWF	0x05
	MOVLW	0x68	;ANSEL(9F)=01101000 Fosc/64=110, analogni vhod AN3=1000
	MOVWF	0x1F
	BCF	0x03,5	;RP0=0 vrnem banko 0
	MOVLW	0x0D	;ADCON0(1F) ADFM=0,VCFG=0,CHS=11,GO/DONE=0,ADON=1 vklop AD in izbira vhoda
	MOVWF	0x1F
	MOVLW	0xFF	;neveljaven kanal (30)
	MOVWF	0x30
	MOVLW	0xFF	;postavi zastavico WDT (35)
	MOVWF	0x35
	CLRF	0x38	;zacetna stevca (38,39) perioda 0F=15 minut
	MOVLW	0x0F
	MOVWF	0x39
	movlw	d'15'	; Timer tiktakanja povzroci prvi "tik" cez 15 x 0,3s
	movwf	led_ticker
	clrf	led_tmo	; Stevec "time out" prizganosti LED "activity"
	CALL	INIREG	;nastavi registre 5,4,3,2,1 MAX2870
	CALL	ADC	;AD pretvornik na vhodu AN3
	CALL	FREQ	;nastavi frekvenco (W) v registru 0 MAX2870
	GOTO	ZANKA

;*** Kuza obnavlja nastavitve *** (W,35,38,39)
KUZA	DECFSZ	0x38,1	;stevec kuza (38) (priblizno minuta)
	RETURN
	DECFSZ	0x39,1	;stevec kuza (39) (stevilo minut)
	RETURN
	CLRF	0x35	;brisi zastavico WDT (35)
	RETURN

;--------------------------------------------------------------------
;*      Glavna programska zanka (+33)                               *
;*      Main program loop                                           *
;--------------------------------------------------------------------
ZANKA	BTFSC	0x35,7		;brisi hardverski WDT?
	CLRWDT
	CALL	KUZA		;kuza obnavlja nastavitve
	CALL	KASNI		;zakasnitev 65536 NOP
	CALL	UNLOCK		;preveri uklenitev zanke
	CALL	ADC		;AD pretvornik na vhodu AN3
	SUBWF	0x30,0		;spremenjen kanal?
	BTFSS	0x03,2		; Ne --> preskoci
	goto	ch_chng		; Da --> zamenjaj kanal
	movf	led_tmo,f	; Preveri stanje led_tmo
	btfsc	STATUS,Z	; Ce led_tmo ni enak 0, preskoci in ukrepaj
	goto	led_ok		; led_tmo enak nic, ukrepanje ni potrebno
	decfsz	led_tmo,f	; led_tmo -- ; ce odstel do 0, preskoci in ugasni LED "activity"
	goto	led_ok		; led_tmo se ni odstel do 0
	bcf	GPIO,led_act	; Ugasni LED
led_ok
	decfsz	led_ticker,f	; LED "activity" tiktak timer --
	goto	led_tckr_ok	; 
	movlw	d'15'		; Ponovno inicializiraj timer tiktakanja na 15 x 0,3s
	movwf	led_ticker
	bsf	GPIO,led_act	; Prizgi LED "activity" za T = 0,3s
	movlw	d'1'		; N = 1 -->
	movwf	led_tmo		; --> led_tmo

led_tckr_ok	
	GOTO	ZANKA
ch_chng	
	CALL	INIREG		; nastavi registre 5,4,3,2,1 MAX2870
	CALL	ADC		; AD pretvornik na vhodu AN3
	MOVWF	0x30
	CALL	FREQ		; nastavi frekvenco (W) v registru 0 MAX2870
	bsf	GPIO,led_act	; Prizgi LED "activity" za T = 2s
	movlw	d'7'		; N = 7 -->
	movwf	led_tmo		; --> led_tmo
	movlw	d'15'		; Reinicializiraj timer tiktakanja na 15 x 0,3s
	movwf	led_ticker
	GOTO	ZANKA

;*** Preveri uklenitev zanke *** (W,30)
UNLOCK	BTFSC	0x05,3
	RETURN
	MOVLW	0xFF	;neveljaven kanal (30)
	MOVWF	0x30
	RETURN

;*** AD pretvornik na vhodu AN3 *** (W,20)
ADC	MOVLW	0x0F	;ADCON0(1F) ADFM=0,VCFG=0,CHS=11,GO/DONE=1,ADON=1
	MOVWF	0x1F
ADC1	BTFSC	0x1F,1	;konec pretvorbe GO/DONE=?
	GOTO	ADC1
	MOVF	0x1E,0	;ADRESH(1E)
	MOVWF	0x20
	RRF	0x20,1	;izlusci gornje 3 bite (20)
	RRF	0x20,1
	RRF	0x20,1
	RRF	0x20,1
	RRF	0x20,1
	MOVLW	0x07
	XORWF	0x20,1
	ANDWF	0x20,1
	INCF	0x20,1	;pretvori v kanal 1-5 (W)
	RRF	0x20,1
	INCF	0x20,1
	ANDWF	0x20,0
	RETURN

;*** Nastavi registre 5,4,3,2,1 MAX2870 *** (W,20-25)
INIREG	MOVLW	0x01	;register5=0x01400005: F01=1,LD=01,MUX3=0
	MOVWF	0x20
	MOVLW	0x40
	MOVWF	0x21
	MOVLW	0x00
	MOVWF	0x22
	MOVLW	0x05
	MOVWF	0x23
	CALL	SPI
	MOVLW	0x00	;register4 = 0x 00 FC 80 3C : BS=0xC8,FB=1,DIVA=b'111',BDIV=0,RFBEN=0,BPWR=0,RFAEN=1, APWR=b'11'(+5dBm)
	MOVWF	0x20
	MOVLW	0xFC
	MOVWF	0x21
	MOVLW	0x80
	MOVWF	0x22
	MOVLW	0x3C
	MOVWF	0x23
	CALL	SPI
	MOVLW	0x00	;register3 = 0x 00 00 08 03 : VCO=000000,VASSHDN=0,RETUNE=0,CDM=00,CDIV = b'0001 0000 0000' = 256 dec
	MOVWF	0x20
	MOVLW	0x00
	MOVWF	0x21
	MOVLW	0x08
	MOVWF	0x22
	MOVLW	0x03
	MOVWF	0x23
	CALL	SPI
	MOVLW	0x78	;register2=0x78007842: LDS=0,SDN=11,MUX=110,DBR=0,RDIV2=0,R=0000000001,REG4DB=1,CP=1100,LDF=0,LDP=0,PDP=1,SHDN=0,TRI=0,RST=0
	MOVWF	0x20
	MOVLW	0x00
	MOVWF	0x21
	MOVLW	0x78
	MOVWF	0x22
	MOVLW	0x42
	MOVWF	0x23
	CALL	SPI
	MOVLW	0x20	;register1=0x20008029: CPOC=0, CPL=01, CPT=00, P=000000000001, M = 5 dec = b'0000 0000 0101'
	MOVWF	0x20
	MOVLW	0x00
	MOVWF	0x21
	MOVLW	0x80
	MOVWF	0x22
	MOVLW	0x29
	MOVWF	0x23
	CALL	SPI
	RETURN

;*** Nastavi frekvenco kanala 0-4 (W) v registru 0 ADF4351 *** (W,20-25)2
FREQ	MOVWF	0x20
	DECFSZ	0x20,1
	GOTO	FREQ1
	MOVLW	0x00	;frekvenca1 = 25.000 MHz = 0x00A00000
	MOVWF	0x20
	MOVLW	0xA0
	MOVWF	0x21
	MOVLW	0x00
	MOVWF	0x22
	MOVLW	0x00
	MOVWF	0x23
	CALL	SPI
	RETURN
FREQ1	DECFSZ	0x20,1
	GOTO	FREQ2
	MOVLW	0x00	;frekvenca2 = 27.000 MHz = 0x00AC8018
	MOVWF	0x20
	MOVLW	0xAC
	MOVWF	0x21
	MOVLW	0x80
	MOVWF	0x22
	MOVLW	0x18
	MOVWF	0x23
	CALL	SPI
	RETURN
FREQ2	DECFSZ	0x20,1
	GOTO	FREQ3
	MOVLW	0x00	;frekvenca3 = 32.000 MHz = 0x00CC8018
	MOVWF	0x20
	MOVLW	0xCC
	MOVWF	0x21
	MOVLW	0x80
	MOVWF	0x22
	MOVLW	0x18
	MOVWF	0x23
	CALL	SPI
	RETURN
FREQ3	DECFSZ	0x20,1
	GOTO	FREQ4
	MOVLW	0x01	;frekvenca4 = 40.000 MHz = 0x01000000
	MOVWF	0x20
	MOVLW	0x00
	MOVWF	0x21
	MOVLW	0x00
	MOVWF	0x22
	MOVLW	0x00
	MOVWF	0x23
	CALL	SPI
	RETURN
FREQ4	NOP
	MOVLW	0x00	;register4 = 0x 00 EC 80 3C : BS=0xC8,FB=1,DIVA=b'110',BDIV=0,RFBEN=0,BPWR=0,RFAEN=1, APWR=b'11' (+5dBm)
	MOVWF	0x20	;(Originalno: register4=0x02 A8 00 24: BS=0x280,FB=1,DIVA=002,BDIV=0,RFBEN=0,BPWR=0,RFAEN=1,APWR=00)
	MOVLW	0xEC
	MOVWF	0x21
	MOVLW	0x80
	MOVWF	0x22
	MOVLW	0x3C
	MOVWF	0x23
	CALL	SPI
	MOVLW	0x00	;frekvenca1 = 25.000 MHz = 0x00A00000
	MOVWF	0x20
	MOVLW	0xA0
	MOVWF	0x21
	MOVLW	0x00
	MOVWF	0x22
	MOVLW	0x00
	MOVWF	0x23
	CALL	SPI
	RETURN

;*** Zakasnitev 65536 NOP *** (W,20,21)
KASNI	MOVLW	0x00
	MOVWF	0x20
	MOVWF	0x21
KASNI1	NOP
	DECFSZ	0x20,1
	GOTO	KASNI1
	DECFSZ	0x21,1
	GOTO	KASNI1
	RETURN

;*** Oddaja 32 bitov iz 20,21,22,23 na SPI *** (W,24,25)
SPI	MOVF	0x20,0	;prvih 8 bitov
	CALL	SPI1
	MOVF	0x21,0	;drugih 8 bitov
	CALL	SPI1
	MOVF	0x22,0	;tretjih 8 bitov
	CALL	SPI1
	MOVF	0x23,0	;zadnjih 8 bitov
	CALL	SPI1
	BCF	0x05,1	;MAX2870 DATA=0 (GP1)
	BSF	0x05,0	;impulz LE MAX2870 (GP0)
	NOP
	BCF	0x05,0
	RETURN
SPI1	MOVWF	0x24	;pomikalni register 24
	MOVLW	0x08	;stevec bitov 25
	MOVWF	0x25
SPI2	BCF	0x05,1	;MAX2870 DATA=0 (GP1)
	BTFSC	0x24,7
	BSF	0x05,1	;MAX2870 DATA=1 (GP1)
	RLF	0x24,1	;zasukaj pomikalni register
	BSF	0x05,2	;impulz MAX2870 (GP2)
	NOP
	BCF	0x05,2
	DECFSZ	0x25,1
	GOTO	SPI2
	RETURN

;********************************************************************
;*                      That's all folks!                           *
;********************************************************************
	end
